/**
   An interface for a rectangular matrix of numerical values.
   @author Frank M. Carrano
   @version 4.0
*/
public interface MatrixInterface 
{
	/** Sets the values in and dimensions of this matrix.
		 @param values  A 2-dimensional array of real values for
					       the corresponding elements in the matrix. */
	public void setMatrix(double[][] values); 
	
	/** Multiplies this matrix by a given matrix.
		 @param anotherMatrix  The second matrix whose number of rows is equal
							        to the number of columns in this matrix.
		 @return  A new matrix that is the product of the two matrices;
				    the original matrices are unchanged. */
	public MatrixInterface multiply(MatrixInterface anotherMatrix);
	
	/** Multiplies this matrix by a given scalar. 
		 @param scalar  The value by which to multiply the matrix.
		 @return  A new matrix that is the product of the matrix and the scalar;
				    the original matrix is unchanged. */
	public MatrixInterface multiply(double scalar);
	
	/** Adds this matrix to a given matrix. 
		 @param anotherMatrix  A second matrix having the same dimensions
		  							as this matrix.
		 @return  A new matrix that is the sum of the two matrices;
				    the original matrices are unchanged. */
	public MatrixInterface add(MatrixInterface anotherMatrix);
	
	/** Transposes this matrix.
		 @return  A new matrix that is the transpose of the matrix;
				    the original matrix is unchanged. */
	public MatrixInterface transpose();  
} // end MatrixInterface
