import java.io.File;
import java.io.FileNotFoundException;
import java.util.Scanner;
/**
   A spelling checker.

   @author Frank M. Carrano
   @version 4.0
 */
public class SpellChecker
{
	BagInterface<String> correctWords;

	public SpellChecker()
	{
		correctWords = new ResizableArrayBag<>();
	} // end default constructor

	/** Initializes the dictionary of correctly spelled words. 
		 @param correctWords  A File object that represents a text file 
							  of correctly spelled words, one per line.
		 @return  True if the dictionary is initialized,
		 		  if not returns false. */
	public boolean setDictionary(File correctWordFile)
	{
		boolean fileOpened = true;
		boolean continueAdding = true;
		try
		{
			Scanner fileData = new Scanner(correctWordFile);
			while (continueAdding && fileData.hasNextLine())
			{
				String word = fileData.nextLine();
				continueAdding = correctWords.add(word);
			} // end while
			fileData.close();
		}
		catch (FileNotFoundException e)
		{
			fileOpened = false; // Error opening the file
		}

		return fileOpened && continueAdding;
	} // end setDictionary

	/** Checks the spelling of a given word. 
		 @param aWord  A string whose spelling is to be checked.
		 @return  True if the given word is spelled correctly,
		 		  otherwise returns false. */
	public boolean checkSpelling(String aWord)
	{
		return correctWords.contains(aWord);
	} // end checkSpelling

} // end SpellChecker
